/*--------------------------------------------------------------------------
 *
 * 3DM-G Interface Software
 *
 * (c) 2003 Microstrain, Inc.
 * All rights reserved.
 *
 * www.microstrain.com
 * 310 Hurricane Lane, Suite 4
 * Williston, VT 05495 USA
 * Tel: 802-862-6629 
 * Fax: 802-863-4093
 *--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------
 * m3dmg.c
 *
 * A test application which excercises the functions of the 3DM-G
 * sensor device. This program should be compiled and linked as follows:
 *
 *  Win32 platforms:
 *      cl m3dmg.c m3dmgAdapter.c m3dmgErrors.c m3dmgSerialWin.c -o m3dmg
 *
 *  Linux platform (with gcc) :
 *      gcc -DLINUX_OS m3dmg.c m3dmgAdapter.c m3dmgErrors.c m3dmgSerialLinux.c -o m3dmg
 *--------------------------------------------------------------------------*/

#include <stdio.h>
#include <windows.h>
#include "m3dmgErrors.h"
#include "m3dmgSerial.h"
#include "m3dmgAdapter.h"

int main(int argc, char **argv) {

    int portNum, portNum2, deviceNum, deviceNum2;
    int i, j;
	BOOL bStopContinuous = FALSE;
    BOOL bQuit = FALSE;

	int iCount = 0;

/* variables used to retieve and store sensor data */
    int sn;
    float temp, ticks;
    float mag[3];       /*  magetic */
    float accel[3];     /*  acceleration */
    float angRate[3];   /*  angular rate */
    char *axis[] = { "X", "Y", "Z"};
    float quat[3];      /*  quaternions */
    float xform[3][3];  /*  transformation matrix */
    float roll, pitch, yaw;
    short address, evalue;  /* EEPROM address and data value */

    float raw[30];

    char *fw;
    int errorCode;
    int tryPortNum = 1;
	int tryPortNum2 = 0;

	printf("3DM-G C-Serial port test\n");

    if (argc > 1) {
        tryPortNum = atoi(argv[1]);
        if (tryPortNum < 1 || tryPortNum > 9) {
            printf("usage:  m3dmg <portNumber> <portNumber2>\n");
            printf("        valid ports are 1..9, default is 1\n");
            exit(1);
		}
		if (argc >2){
          tryPortNum2 = atoi(argv[2]);
          if (tryPortNum2 < 1 || tryPortNum2 > 9) {
              printf("usage:  m3dmg <portNumber> <portNumber2>\n");
              printf("        valid ports are 1..9, default is 1\n");
              exit(1);
		  }
		}
    }

    /* open a port, map a device */
    portNum = m3dmg_openPort(tryPortNum, 38400, 8, 0, 1);
    if (portNum<0) {
        printf("port open failed.\n");
        printf("Comm error %d, %s: ", portNum, explainError(portNum));
        return -1;
    }
    printf("Port is open as #%d - OK.\n", portNum);

	if(tryPortNum2 != 0){
	   portNum2 = m3dmg_openPort(tryPortNum2, 38400, 8, 0, 1);
       if (portNum2<0) {
           printf("port 2 open failed.\n");
           printf("Comm error %d, %s: ", portNum, explainError(portNum));
           return -1;
       }
     printf("Port 2 is open as #%d - OK.\n", portNum);
	}

    /* map device - this is required! */
    deviceNum = m3dmg_mapDevice(1, portNum);
    if (deviceNum <= 0) {
        printf("Could not map the Device to a Port\n");
        printf("%s\n", explainError(deviceNum));
        return -1;
    }
    printf("Device Number is #: %d\n\n", deviceNum);

	 if (portNum2 > 0) {
	     deviceNum2 = m3dmg_mapDevice(2, portNum2);
        if (deviceNum2 <= 0) {
			printf("Could not map the Device to a Port port #2\n");
            printf("%s\n", explainError(deviceNum2));
            return -1;
       }
       printf("Device Number 2 is #: %d\n\n", deviceNum2);
	 }

    /* get serial number */
    errorCode = m3dmg_getSerialNumber(deviceNum, &sn);
    if (errorCode != M3D_OK)
        printf("S/N Error - : %s\n", explainError(errorCode));
    else
        printf("Serial #: %d\n", sn);
    printf("\n");

    /* get firmware number (as a string) */
    fw = (char *) malloc( (size_t)12 );
    errorCode = m3dmg_getFirmwareVersion(deviceNum, fw);
    printf("Firmware ");
    if (errorCode < 0)
        printf("Error - : %s\n", explainError(errorCode));
    else
        printf("#: %s\n", fw);
    printf("\n");

    

  
    printf("Reading streaming data (hit s to Stop streaming).\n");
    while(!bStopContinuous){
       /* quaternions and vectors together (gyro-stabilized) */
       errorCode = m3dmg_getGyroStabQuatVectors(deviceNum, 
                                           quat, 
                                           mag, accel, angRate);
       printf("Stabilized Quaternions & Vectors\n");
       if (errorCode < 0)
           printf("Error - : %s\n", explainError(errorCode));
       else {
           for (i=0; i<4; i++) {
               printf("  %d : %6.4f\n", i, quat[i]);
           }
           printf("\tMag\t\tAccel\t\tAng Rate\n");
           for (i=0; i<3; i++) {
              printf("  %s\t: %f\t%f\t%f\n", axis[i], mag[i], accel[i], angRate[i]);
           }
       }
	   if (deviceNum2 >0){
           errorCode = m3dmg_getGyroStabQuatVectors(deviceNum2, 
                                           quat, 
                                           mag, accel, angRate);
           printf("Stabilized Quaternions & Vectors from device 2\n");
           if (errorCode < 0)
               printf("Error - : %s\n", explainError(errorCode));
           else {
                for (i=0; i<4; i++) {
                    printf("  %d : %6.4f\n", i, quat[i]);
                }
                printf("\tMag\t\tAccel\t\tAng Rate\n");
                for (i=0; i<3; i++) {
                     printf("  %s\t: %f\t%f\t%f\n", axis[i], mag[i], accel[i], angRate[i]);
                }
           }
	   }
	   //check for a key every 50 itterations
	   if(iCount++ > 50){
			int ch = 0;
			if(ReadCharNoReturn(&ch)){
				bStopContinuous = (ch == 's' || ch == 'S');
			}
			//rest the counter
			iCount = 0;
		}
	}/*end while loop */
    printf("\n");


    /* free allocated memory */
    free(fw);

    /* close device */
    m3dmg_closeDevice(deviceNum);
	if(deviceNum2 >0)
	   m3dmg_closeDevice(deviceNum2);
	return(0);
}

/*-------------- end of m3dmg.c ----------------------*/
