/*--------------------------------------------------------------------------
 *
 * 3DM-G Interface Software
 *
 * (c) 2003 Microstrain, Inc.
 * All rights reserved.
 *
 * www.microstrain.com
 * 310 Hurricane Lane, Suite 4
 * Williston, VT 05495 USA
 * Tel: 802-862-6629 
 * Fax: 802-863-4093
 *--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------
 * m3dmgbaud.c
 *
 * A test application which will change the baudrate for a sensor device.
 * The sensor must be turned off, then on again for the change to take effect.
 * (cycle the power).
 *
 *  Win32 platforms:
 *  cl m3dmgbaud.c m3dmgAdapter.c m3dmgErrors.c m3dmgSerialWin.c -o m3dmgbaud
 *
 *  Linux platform (with gcc) :
 *  gcc -DLINUX_OS m3dmgbaud.c m3dmgAdapter.c m3dmgErrors.c m3dmgSerialLinux.c -o m3dmgbaud
 *--------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include "m3dmgErrors.h"
#include "m3dmgSerial.h"
#include "m3dmgAdapter.h"
#include "m3dmgUtils.h"

int main(int argc, char **argv) {

    int portNum, deviceNum;

/* variables used to retieve and store sensor data */
    int sn;
    char *fw;
    int oldRate, newRate;
    int errorCode;
    short address, value;
    int tryPortNum = 1;

	printf("3DM-G Baud Rate Change\n");
    if (argc<3) {
        printf("usage: m3dmgbaud <oldrate> <newrate> [portNumber]\n");
        printf("       port number is optional (defaults to 1)\n");
        exit(-1);
    }

    oldRate = atoi(argv[1]);
    newRate = atoi(argv[2]);
    printf("From: %d to %d\n", oldRate, newRate);

    if (!(newRate==19200 || newRate==38400 || newRate==115200)) {
        printf("Only these rates are supported: 19200, 38400, 115200\n");
        exit(-1);
    }

    if (argc == 4) {
        tryPortNum = atoi(argv[3]);
        if (tryPortNum < 1 || tryPortNum > 9) {
            printf("usage: m3dmgbaud <oldrate> <newrate> [portNumber]\n");
            printf("        valid ports are 1..9, default is 1\n");
            exit(1);
        }
    }




    /* open a port, map a device */
    portNum = m3dmg_openPort(tryPortNum, oldRate, 8, 0, 1);
    if (portNum<0) {
        printf("port open failed.\n");
        printf("Comm error %d, %s: ", portNum, explainError(portNum));
        return -1;
    }
    printf("Port is open as #%d - OK.\n", portNum);

    /* map device - this is required! */
    deviceNum = m3dmg_mapDevice(1, portNum);
    if (deviceNum <= 0) {
        printf("Could not map the Device to a Port\n");
        printf("%s\n", explainError(deviceNum));
        return -1;
    }

    /* get serial number */
    errorCode = m3dmg_getSerialNumber(deviceNum, &sn);
    if (errorCode != M3D_OK) {
        printf("S/N Error - : %s\n", explainError(errorCode));
        exit(-1);
    }
    printf("Serial #: %d\n", sn);

    /* get firmware number (as a string) */
    fw = (char *) malloc( (size_t)12 );
    errorCode = m3dmg_getFirmwareVersion(deviceNum, fw);
    printf("Firmware ");
    if (errorCode < 0)
        printf("Error - : %s\n", explainError(errorCode));
    else
        printf("#: %s\n", fw);

    /* set the baud rate */
    address = 44;
    value = 64;    /* 38400 rate by defaule */
    if (newRate==19200)
        value = 129;
    else if (newRate==38400)
        value = 64;
    else if (newRate==115200)
        value = 21;

    errorCode = m3dmg_setEEPROMValue(deviceNum, address, &value);
    printf("Set baud rate: %d code: %d\n", newRate, value);

    if (errorCode < 0)
        printf("Error - : %s\n", explainError(errorCode));
    else {
        printf("The new baud rate only takes effect\n");
        printf("after the sensor power is cycled. (off->on)\n");
    }

    printf("\n");

    /* free allocated memory */
    free(fw);

    /* close device */
    m3dmg_closeDevice(deviceNum);
	return(0);
}

/*-------------- end of m3dmgbaud.c ----------------------*/
